# -*- coding: utf-8 -*-
"""Common parts to all other modules, mainly utility functions.
"""
import pandas as pd

TYPE_CAT = 'CAT'
"""String: A categorical variable"""

TYPE_BOOL = 'BOOL'
"""String: A boolean variable"""

TYPE_NUM = 'NUM'
"""String: A numerical variable"""

TYPE_DATE = 'DATE'
"""String: A numeric variable"""

S_TYPE_CONST = 'CONST'
"""String: A constant variable"""

S_TYPE_UNIQUE = 'UNIQUE'
"""String: A unique variable"""

S_TYPE_UNSUPPORTED = 'UNSUPPORTED'
"""String: An unsupported variable"""

_VALUE_COUNTS_MEMO = {}

def get_groupby_statistic(data):
    """Calculate value counts and distinct count of a variable (technically a Series).

    The result is cached by column name in a global variable to avoid recomputing.

    Parameters
    ----------
    data : Series
        The data type of the Series.

    Returns
    -------
    list
        value count and distinct count
    """
    if data.name is not None and data.name in _VALUE_COUNTS_MEMO:
        return _VALUE_COUNTS_MEMO[data.name]

    value_counts_with_nan = data.value_counts(dropna=False)
    value_counts_without_nan = value_counts_with_nan.loc[value_counts_with_nan.index.dropna()]
    distinct_count_with_nan = value_counts_with_nan.count()

    # When the inferred type of the index is just "mixed" probably the types within the series are tuple, dict, list and so on...
    if value_counts_without_nan.index.inferred_type == "mixed":
        raise TypeError('Not supported mixed type')

    result = [value_counts_without_nan, distinct_count_with_nan]

    if data.name is not None:
        _VALUE_COUNTS_MEMO[data.name] = result

    return result

_MEMO = {}
def get_vartype(data):
    """Infer the type of a variable (technically a Series).

    The types supported are split in standard types and special types.

    Standard types:
        * Categorical (`TYPE_CAT`): the default type if no other one can be determined
        * Numerical (`TYPE_NUM`): if it contains numbers
        * Boolean (`TYPE_BOOL`): at this time only detected if it contains boolean values, see todo
        * Date (`TYPE_DATE`): if it contains datetime

    Special types:
        * Constant (`S_TYPE_CONST`): if all values in the variable are equal
        * Unique (`S_TYPE_UNIQUE`): if all values in the variable are different
        * Unsupported (`S_TYPE_UNSUPPORTED`): if the variable is unsupported

     The result is cached by column name in a global variable to avoid recomputing.

    Parameters
    ----------
    data : Series
        The data type of the Series.

    Returns
    -------
    str
        The data type of the Series.

    Notes
    ----
        * Should improve verification when a categorical or numeric field has 3 values, it could be a categorical field
        or just a boolean with NaN values
        * #72: Numeric with low Distinct count should be treated as "Categorical"
    """
    if data.name is not None and data.name in _MEMO:
        return _MEMO[data.name]

    vartype = None
    try:
        distinct_count = get_groupby_statistic(data)[1]
        leng = len(data)

        if distinct_count <= 1:
            vartype = S_TYPE_CONST
        elif pd.api.types.is_bool_dtype(data) or (distinct_count == 2 and pd.api.types.is_numeric_dtype(data)):
            vartype = TYPE_BOOL
        elif pd.api.types.is_numeric_dtype(data):
            vartype = TYPE_NUM
        elif pd.api.types.is_datetime64_dtype(data):
            vartype = TYPE_DATE
        elif distinct_count == leng:
            vartype = S_TYPE_UNIQUE
        else:
            vartype = TYPE_CAT
    except:
        vartype = S_TYPE_UNSUPPORTED

    if data.name is not None:
        _MEMO[data.name] = vartype

    return vartype

def clear_cache():
    """Clear the cache stored as global variables"""
    global _MEMO, _VALUE_COUNTS_MEMO
    _MEMO = {}
    _VALUE_COUNTS_MEMO = {}



# -*- coding: utf-8 -*-
"""Plot distribution of datasets"""

import base64
from distutils.version import LooseVersion
import pandas_profiling.base as base
import matplotlib
import numpy as np
# Fix #68, this call is not needed and brings side effects in some use cases
# Backend name specifications are not case-sensitive; e.g., ‘GTKAgg’ and ‘gtkagg’ are equivalent.
# See https://matplotlib.org/faq/usage_faq.html#what-is-a-backend
BACKEND = 'Agg'
if matplotlib.get_backend().lower() != BACKEND.lower():
    # If backend is not set properly a call to describe will hang
    matplotlib.use(BACKEND)
from matplotlib import pyplot as plt
try:
    from StringIO import BytesIO
except ImportError:
    from io import BytesIO
try:
    from urllib import quote
except ImportError:
    from urllib.parse import quote

def _plot_histogram(series, bins=10, figsize=(6, 4), facecolor='#337ab7'):
    """Plot an histogram from the data and return the AxesSubplot object.

    Parameters
    ----------
    series : Series
        The data to plot
    figsize : tuple
        The size of the figure (width, height) in inches, default (6,4)
    facecolor : str
        The color code.

    Returns
    -------
    matplotlib.AxesSubplot
        The plot.
    """
    if base.get_vartype(series) == base.TYPE_DATE:
        # TODO: These calls should be merged
        fig = plt.figure(figsize=figsize)
        plot = fig.add_subplot(111)
        plot.set_ylabel('Frequency')
        try:
            plot.hist(series.values, facecolor=facecolor, bins=bins)
        except TypeError: # matplotlib 1.4 can't plot dates so will show empty plot instead
            pass
    else:
        plot = series.plot(kind='hist', figsize=figsize,
                           facecolor=facecolor,
                           bins=bins)  # TODO when running on server, send this off to a different thread
    return plot


def histogram(series, **kwargs):
    """Plot an histogram of the data.

    Parameters
    ----------
    series: Series
        The data to plot.

    Returns
    -------
    str
        The resulting image encoded as a string.
    """
    imgdata = BytesIO()
    plot = _plot_histogram(series, **kwargs)
    plot.figure.subplots_adjust(left=0.15, right=0.95, top=0.9, bottom=0.1, wspace=0, hspace=0)
    plot.figure.savefig(imgdata)
    imgdata.seek(0)
    result_string = 'data:image/png;base64,' + quote(base64.b64encode(imgdata.getvalue()))
    # TODO Think about writing this to disk instead of caching them in strings
    plt.close(plot.figure)
    return result_string


def mini_histogram(series, **kwargs):
    """Plot a small (mini) histogram of the data.

    Parameters
    ----------
    series: Series
        The data to plot.

    Returns
    -------
    str
        The resulting image encoded as a string.
    """
    imgdata = BytesIO()
    plot = _plot_histogram(series, figsize=(2, 0.75), **kwargs)
    plot.axes.get_yaxis().set_visible(False)

    if LooseVersion(matplotlib.__version__) <= '1.5.9':
        plot.set_axis_bgcolor("w")
    else:
        plot.set_facecolor("w")

    xticks = plot.xaxis.get_major_ticks()
    for tick in xticks[1:-1]:
        tick.set_visible(False)
        tick.label.set_visible(False)
    for tick in (xticks[0], xticks[-1]):
        tick.label.set_fontsize(8)
    plot.figure.subplots_adjust(left=0.15, right=0.85, top=1, bottom=0.35, wspace=0, hspace=0)
    plot.figure.savefig(imgdata)
    imgdata.seek(0)
    result_string = 'data:image/png;base64,' + quote(base64.b64encode(imgdata.getvalue()))
    plt.close(plot.figure)
    return result_string

def correlation_matrix(corrdf, title, **kwargs):
    """Plot image of a matrix correlation.
    Parameters
    ----------
    corrdf: DataFrame
        The matrix correlation to plot.
    title: str
        The matrix title
    Returns
    -------
    str, The resulting image encoded as a string.
    """
    imgdata = BytesIO()
    fig_cor, axes_cor = plt.subplots(1, 1)
    labels = corrdf.columns
    matrix_image = axes_cor.imshow(corrdf, vmin=-1, vmax=1, interpolation="nearest", cmap='bwr')
    plt.title(title, size=18)
    plt.colorbar(matrix_image)
    axes_cor.set_xticks(np.arange(0, corrdf.shape[0], corrdf.shape[0] * 1.0 / len(labels)))
    axes_cor.set_yticks(np.arange(0, corrdf.shape[1], corrdf.shape[1] * 1.0 / len(labels)))
    axes_cor.set_xticklabels(labels, rotation=90)
    axes_cor.set_yticklabels(labels)

    matrix_image.figure.savefig(imgdata, bbox_inches='tight')
    imgdata.seek(0)
    result_string = 'data:image/png;base64,' + quote(base64.b64encode(imgdata.getvalue()))
    plt.close(matrix_image.figure)
    return result_string


# -*- coding: utf-8 -*-
"""Compute statistical description of datasets"""
import multiprocessing
import itertools
from functools import partial
import numpy as np
import pandas as pd
import matplotlib

from pkg_resources import resource_filename
import pandas_profiling.formatters as formatters
import pandas_profiling.base as base
from pandas_profiling.plot import histogram, mini_histogram

def describe_numeric_1d(series, **kwargs):
    """Compute summary statistics of a numerical (`TYPE_NUM`) variable (a Series).

    Also create histograms (mini an full) of its distribution.

    Parameters
    ----------
    series : Series
        The variable to describe.

    Returns
    -------
    Series
        The description of the variable as a Series with index being stats keys.
    """
    # Format a number as a percentage. For example 0.25 will be turned to 25%.
    _percentile_format = "{:.0%}"
    stats = dict()
    stats['type'] = base.TYPE_NUM
    stats['mean'] = series.mean()
    stats['std'] = series.std()
    stats['variance'] = series.var()
    stats['min'] = series.min()
    stats['max'] = series.max()
    stats['range'] = stats['max'] - stats['min']
    # To avoid to compute it several times
    _series_no_na = series.dropna()
    for percentile in np.array([0.05, 0.25, 0.5, 0.75, 0.95]):
        # The dropna() is a workaround for https://github.com/pydata/pandas/issues/13098
        stats[_percentile_format.format(percentile)] = _series_no_na.quantile(percentile)
    stats['iqr'] = stats['75%'] - stats['25%']
    stats['kurtosis'] = series.kurt()
    stats['skewness'] = series.skew()
    stats['sum'] = series.sum()
    stats['mad'] = series.mad()
    stats['cv'] = stats['std'] / stats['mean'] if stats['mean'] else np.NaN
    stats['n_zeros'] = (len(series) - np.count_nonzero(series))
    stats['p_zeros'] = stats['n_zeros'] / len(series)
    # Histograms
    stats['histogram'] = histogram(series, **kwargs)
    stats['mini_histogram'] = mini_histogram(series, **kwargs)
    return pd.Series(stats, name=series.name)


def describe_date_1d(series):
    """Compute summary statistics of a date (`TYPE_DATE`) variable (a Series).

    Also create histograms (mini an full) of its distribution.

    Parameters
    ----------
    series : Series
        The variable to describe.

    Returns
    -------
    Series
        The description of the variable as a Series with index being stats keys.
    """
    stats = dict()
    stats['type'] = base.TYPE_DATE
    stats['min'] = series.min()
    stats['max'] = series.max()
    stats['range'] = stats['max'] - stats['min']
    # Histograms
    stats['histogram'] = histogram(series)
    stats['mini_histogram'] = mini_histogram(series)
    return pd.Series(stats, name=series.name)

def describe_categorical_1d(series):
    """Compute summary statistics of a categorical (`TYPE_CAT`) variable (a Series).

    Parameters
    ----------
    series : Series
        The variable to describe.

    Returns
    -------
    Series
        The description of the variable as a Series with index being stats keys.
    """
    # Only run if at least 1 non-missing value
    value_counts, distinct_count = base.get_groupby_statistic(series)
    top, freq = value_counts.index[0], value_counts.iloc[0]
    names = []
    result = []

    if base.get_vartype(series) == base.TYPE_CAT:
        names += ['top', 'freq', 'type']
        result += [top, freq, base.TYPE_CAT]

    return pd.Series(result, index=names, name=series.name)

def describe_boolean_1d(series):
    """Compute summary statistics of a boolean (`TYPE_BOOL`) variable (a Series).

    Parameters
    ----------
    series : Series
        The variable to describe.

    Returns
    -------
    Series
        The description of the variable as a Series with index being stats keys.
    """
    value_counts, distinct_count = base.get_groupby_statistic(series)
    top, freq = value_counts.index[0], value_counts.iloc[0]
    # The mean of boolean is an interesting information
    mean = series.mean()
    names = []
    result = []
    names += ['top', 'freq', 'type', 'mean']
    result += [top, freq, base.TYPE_BOOL, mean]

    return pd.Series(result, index=names, name=series.name)

def describe_constant_1d(series):
    """Compute summary statistics of a constant (`S_TYPE_CONST`) variable (a Series).

    Parameters
    ----------
    series : Series
        The variable to describe.

    Returns
    -------
    Series
        The description of the variable as a Series with index being stats keys.
    """
    return pd.Series([base.S_TYPE_CONST], index=['type'], name=series.name)

def describe_unique_1d(series):
    """Compute summary statistics of a unique (`S_TYPE_UNIQUE`) variable (a Series).

    Parameters
    ----------
    series : Series
        The variable to describe.

    Returns
    -------
    Series
        The description of the variable as a Series with index being stats keys.
    """
    return pd.Series([base.S_TYPE_UNIQUE], index=['type'], name=series.name)

def describe_supported(series, **kwargs):
    """Compute summary statistics of a supported variable (a Series).

    Parameters
    ----------
    series : Series
        The variable to describe.

    Returns
    -------
    Series
        The description of the variable as a Series with index being stats keys.
    """
    leng = len(series)  # number of observations in the Series
    count = series.count()  # number of non-NaN observations in the Series
    n_infinite = count - series.count()  # number of infinte observations in the Series

    value_counts, distinct_count = base.get_groupby_statistic(series)
    if count > distinct_count > 1:
        mode = series.mode().iloc[0]
    else:
        mode = series[0]

    results_data = {'count': count,
                    'distinct_count': distinct_count,
                    'p_missing': 1 - count / leng,
                    'n_missing': leng - count,
                    'p_infinite': n_infinite / leng,
                    'n_infinite': n_infinite,
                    'is_unique': distinct_count == leng,
                    'mode': mode,
                    'p_unique': distinct_count / leng}
    try:
        # pandas 0.17 onwards
        results_data['memorysize'] = series.memory_usage()
    except:
        results_data['memorysize'] = 0

    return pd.Series(results_data, name=series.name)

def describe_unsupported(series, **kwargs):
    """Compute summary statistics of a unsupported (`S_TYPE_UNSUPPORTED`) variable (a Series).

    Parameters
    ----------
    series : Series
        The variable to describe.

    Returns
    -------
    Series
        The description of the variable as a Series with index being stats keys.
    """
    leng = len(series)  # number of observations in the Series
    count = series.count()  # number of non-NaN observations in the Series
    n_infinite = count - series.count()  # number of infinte observations in the Series

    results_data = {'count': count,
                    'p_missing': 1 - count / leng,
                    'n_missing': leng - count,
                    'p_infinite': n_infinite / leng,
                    'n_infinite': n_infinite,
                    'type': base.S_TYPE_UNSUPPORTED}

    try:
        # pandas 0.17 onwards
        results_data['memorysize'] = series.memory_usage()
    except:
        results_data['memorysize'] = 0

    return pd.Series(results_data, name=series.name)

def describe_1d(data, **kwargs):
    """Compute summary statistics of a variable (a Series).

    The description is different according to the type of the variable.
    However a set of common stats is also computed.

    Parameters
    ----------
    series : Series
        The variable to describe.

    Returns
    -------
    Series
        The description of the variable as a Series with index being stats keys.
    """

    # Replace infinite values with NaNs to avoid issues with
    # histograms later.
    data.replace(to_replace=[np.inf, np.NINF, np.PINF], value=np.nan, inplace=True)

    result = pd.Series({}, name=data.name)

    vartype = base.get_vartype(data)

    if vartype == base.S_TYPE_UNSUPPORTED:
        result = result.append(describe_unsupported(data))
    else:
        result = result.append(describe_supported(data))

        if vartype == base.S_TYPE_CONST:
            result = result.append(describe_constant_1d(data))
        elif vartype == base.TYPE_BOOL:
            result = result.append(describe_boolean_1d(data, **kwargs))
        elif vartype == base.TYPE_NUM:
            result = result.append(describe_numeric_1d(data, **kwargs))
        elif vartype == base.TYPE_DATE:
            result = result.append(describe_date_1d(data, **kwargs))
        elif vartype == base.S_TYPE_UNIQUE:
            result = result.append(describe_unique_1d(data, **kwargs))
        else:
            # TYPE_CAT
            result = result.append(describe_categorical_1d(data))

    return result

def multiprocess_func(x, **kwargs):
    return x[0], describe_1d(x[1], **kwargs)

def describe(df, bins=10, check_correlation=True, correlation_threshold=0.9, correlation_overrides=None, check_recoded=False, pool_size=multiprocessing.cpu_count(), **kwargs):
    """Generates a dict containing summary statistics for a given dataset stored as a pandas `DataFrame`.

    Used has is it will output its content as an HTML report in a Jupyter notebook.

    Parameters
    ----------
    df : DataFrame
        Data to be analyzed
    bins : int
        Number of bins in histogram.
        The default is 10.
    check_correlation : boolean
        Whether or not to check correlation.
        It's `True` by default.
    correlation_threshold: float
        Threshold to determine if the variable pair is correlated.
        The default is 0.9.
    correlation_overrides : list
        Variable names not to be rejected because they are correlated.
        There is no variable in the list (`None`) by default.
    check_recoded : boolean
        Whether or not to check recoded correlation (memory heavy feature).
        Since it's an expensive computation it can be activated for small datasets.
        `check_correlation` must be true to disable this check.
        It's `False` by default.
    pool_size : int
        Number of workers in thread pool
        The default is equal to the number of CPU.

    Returns
    -------
    dict
        Containing the following keys:
            * table: general statistics on the dataset
            * variables: summary statistics for each variable
            * freq: frequency table

    Notes:
    ------
        * The section dedicated to check the correlation should be externalized
    """

    if not isinstance(df, pd.DataFrame):
        raise TypeError("df must be of type pandas.DataFrame")
    if df.empty:
        raise ValueError("df can not be empty")

    try:
        # reset matplotlib style before use
        # Fails in matplotlib 1.4.x so plot might look bad
        matplotlib.style.use("default")
    except:
        pass

    matplotlib.style.use(resource_filename(__name__, "pandas_profiling.mplstyle"))
    
    # Clearing the cache before computing stats
    base.clear_cache()

    if not pd.Index(np.arange(0, len(df))).equals(df.index):
        # Treat index as any other column
        df = df.reset_index()

    # Describe all variables in a univariate way
    pool = multiprocessing.Pool(pool_size)
    local_multiprocess_func = partial(multiprocess_func, **kwargs)
    ldesc = {col: s for col, s in pool.map(local_multiprocess_func, df.iteritems())}
    pool.close()

    # Get correlations
    dfcorrPear = df.corr(method="pearson")
    dfcorrSpear = df.corr(method="spearman")

    # Check correlations between variable
    if check_correlation is True:
        ''' TODO: corr(x,y) > 0.9 and corr(y,z) > 0.9 does not imply corr(x,z) > 0.9
        If x~y and y~z but not x~z, it would be better to delete only y
        Better way would be to find out which variable causes the highest increase in multicollinearity.
        '''
        corr = dfcorrPear.copy()
        for x, corr_x in corr.iterrows():
            if correlation_overrides and x in correlation_overrides:
                continue

            for y, corr in corr_x.iteritems():
                if x == y: break

                if corr > correlation_threshold:
                    ldesc[x] = pd.Series(['CORR', y, corr], index=['type', 'correlation_var', 'correlation'])

        if check_recoded:
            categorical_variables = [(name, data) for (name, data) in df.iteritems() if base.get_vartype(data)=='CAT']
            for (name1, data1), (name2, data2) in itertools.combinations(categorical_variables, 2):
                if correlation_overrides and name1 in correlation_overrides:
                    continue

                confusion_matrix=pd.crosstab(data1,data2)
                if confusion_matrix.values.diagonal().sum() == len(df):
                    ldesc[name1] = pd.Series(['RECODED', name2], index=['type', 'correlation_var'])

    # Convert ldesc to a DataFrame
    names = []
    ldesc_indexes = sorted([x.index for x in ldesc.values()], key=len)
    for idxnames in ldesc_indexes:
        for name in idxnames:
            if name not in names:
                names.append(name)
    variable_stats = pd.concat(ldesc, join_axes=pd.Index([names]), axis=1)
    variable_stats.columns.names = df.columns.names

    # General statistics
    table_stats = {}

    table_stats['n'] = len(df)
    table_stats['nvar'] = len(df.columns)
    table_stats['total_missing'] = variable_stats.loc['n_missing'].sum() / (table_stats['n'] * table_stats['nvar'])
    unsupported_columns = variable_stats.transpose()[variable_stats.transpose().type != base.S_TYPE_UNSUPPORTED].index.tolist()
    table_stats['n_duplicates'] = sum(df.duplicated(subset=unsupported_columns)) if len(unsupported_columns) > 0 else 0

    memsize = df.memory_usage(index=True).sum()
    table_stats['memsize'] = formatters.fmt_bytesize(memsize)
    table_stats['recordsize'] = formatters.fmt_bytesize(memsize / table_stats['n'])

    table_stats.update({k: 0 for k in ("NUM", "DATE", "CONST", "CAT", "UNIQUE", "CORR", "RECODED", "BOOL", "UNSUPPORTED")})
    table_stats.update(dict(variable_stats.loc['type'].value_counts()))
    table_stats['REJECTED'] = table_stats['CONST'] + table_stats['CORR'] + table_stats['RECODED']

    return {
        'table': table_stats,
        'variables': variable_stats.T,
        'freq': {k: (base.get_groupby_statistic(df[k])[0] if variable_stats[k].type != base.S_TYPE_UNSUPPORTED else None) for k in df.columns},
        'correlations': {'pearson': dfcorrPear, 'spearman': dfcorrSpear}
    }



# coding=UTF-8
"""Contains all templates used for generating the HTML profile report"""

from jinja2 import Environment, PackageLoader

# Initializing Jinja
pl = PackageLoader('pandas_profiling', 'templates')
jinja2_env = Environment(lstrip_blocks=True, trim_blocks=True, loader=pl)

# Mapping between template name and file
templates = {'freq_table_row': 'freq_table_row.html',
             'mini_freq_table_row': 'mini_freq_table_row.html',
             'freq_table': 'freq_table.html',
             'mini_freq_table': 'mini_freq_table.html',
             'row_num': 'row_num.html',
             'row_date': 'row_date.html',
             'row_cat': 'row_cat.html',
             'row_bool': 'row_bool.html',
             'row_corr': 'row_corr.html',
             'row_recoded': 'row_recoded.html',
             'row_const': 'row_const.html',
             'row_unique': 'row_unique.html',
             'row_unsupported': 'row_unsupported.html',
             'overview': 'overview.html',
             'sample': 'sample.html',
             'base': 'base.html',
             'wrapper': 'wrapper.html',
             'correlations' : 'correlations.html'
             }

# Mapping between row type and var type
var_type = {'NUM': 'Numeric',
            'DATE': 'Date',
            'CAT': 'Categorical',
            'UNIQUE': 'Categorical, Unique',
            'BOOL': 'Boolean',
            'CONST': 'Constant',
            'CORR': 'Highly correlated',
            'RECODED': 'Recoded',
            'UNSUPPORTED': 'Unsupported'
            }


def template(template_name):
    """Return a jinja template ready for rendering. If needed, global variables are initialized.

    Parameters
    ----------
    template_name: str, the name of the template as defined in the templates mapping

    Returns
    -------
    The Jinja template ready for rendering
    """
    globals = None
    if template_name.startswith('row_'):
        # This is a row template setting global variable
        globals = dict()
        globals['vartype'] = var_type[template_name.split('_')[1].upper()]
    return jinja2_env.get_template(templates[template_name], globals=globals)


# mapping between row type and template name
row_templates_dict = {'NUM': template('row_num'),
                      'DATE': template('row_date'),
                      'DISCRETE': template('row_num'),
                      'CAT': template('row_cat'),
                      'BOOL': template('row_bool'),
                      'UNIQUE': template('row_unique'),
                      'CONST': template('row_const'),
                      'CORR': template('row_corr'),
                      'RECODED': template('row_recoded'),
                      'UNSUPPORTED': template('row_unsupported')
                      }

# The number of column to use in the display of the frequency table according to the category
mini_freq_table_nb_col = {'CAT': 6, 'BOOL': 3}

messages = dict()
messages['CONST'] = u'<a href="#pp_var_{0[varname]}"><code>{0[varname]}</code></a> has constant value {0[mode]} <span class="label label-primary">Rejected</span>'
messages['CORR'] = u'<a href="#pp_var_{0[varname]}"><code>{0[varname]}</code></a> is highly correlated with <a href="#pp_var_{0[correlation_var]}"><code>{0[correlation_var]}</code></a> (ρ = {0[correlation]}) <span class="label label-primary">Rejected</span>'
messages['RECODED'] = u'<a href="#pp_var_{0[varname]}"><code>{0[varname]}</code></a> is a recoding of <a href="#pp_var_{0[correlation_var]}"><code>{0[correlation_var]}</code></a> <span class="label label-primary">Rejected</span>'
messages['HIGH_CARDINALITY'] = u'<a href="#pp_var_{0[varname]}"><code>{varname}</code></a> has a high cardinality: {0[distinct_count]} distinct values  <span class="label label-warning">Warning</span>'
messages['UNSUPPORTED'] = u'<a href="#pp_var_{0[varname]}"><code>{0[varname]}</code></a> is an unsupported type, check if it needs cleaning or further analysis <span class="label label-warning">Warning</span>'
messages['n_duplicates'] = u'Dataset has {0[n_duplicates]} duplicate rows <span class="label label-warning">Warning</span>'
messages['skewness'] = u'<a href="#pp_var_{0[varname]}"><code>{varname}</code></a> is highly skewed (γ1 = {0[skewness]})  <span class="label label-info">Skewed</span>'
messages['p_missing'] = u'<a href="#pp_var_{0[varname]}"><code>{varname}</code></a> has {0[n_missing]} / {0[p_missing]} missing values <span class="label label-default">Missing</span>'
messages['p_infinite'] = u'<a href="#pp_var_{0[varname]}"><code>{varname}</code></a> has {0[n_infinite]} / {0[p_infinite]} infinite values <span class="label label-default">Infinite</span>'
messages['p_zeros'] = u'<a href="#pp_var_{0[varname]}"><code>{varname}</code></a> has {0[n_zeros]} / {0[p_zeros]} zeros <span class="label label-info">Zeros</span>'

message_row = u'<li>{message}</li>'



# -*- coding: utf-8 -*-
"""Main module of pandas-profiling.

Docstring is compliant with NumPy/SciPy documentation standard:
https://github.com/numpy/numpy/blob/master/doc/HOWTO_DOCUMENT.rst.txt
See also for a short description of docstring:
https://stackoverflow.com/questions/3898572/what-is-the-standard-python-docstring-format
"""
import codecs
import pandas_profiling.templates as templates
from .describe import describe
from .report import to_html

NO_OUTPUTFILE = "pandas_profiling.no_outputfile"
DEFAULT_OUTPUTFILE = "pandas_profiling.default_outputfile"

class ProfileReport(object):
    """Generate a profile report from a Dataset stored as a pandas `DataFrame`.

    Used has is it will output its content as an HTML report in a Jupyter notebook.

    Attributes
    ----------
    df : DataFrame
        Data to be analyzed
    bins : int
        Number of bins in histogram.
        The default is 10.
    check_correlation : boolean
        Whether or not to check correlation.
        It's `True` by default.
    correlation_threshold: float
        Threshold to determine if the variable pair is correlated.
        The default is 0.9.
    correlation_overrides : list
        Variable names not to be rejected because they are correlated.
        There is no variable in the list (`None`) by default.
    check_recoded : boolean
        Whether or not to check recoded correlation (memory heavy feature).
        Since it's an expensive computation it can be activated for small datasets.
        `check_correlation` must be true to disable this check.
        It's `False` by default.
    pool_size : int
        Number of workers in thread pool
        The default is equal to the number of CPU.

    Methods
    -------
    get_description
        Return the description (a raw statistical summary) of the dataset.
    get_rejected_variables
        Return the list of rejected variable or an empty list if there is no rejected variables.
    to_file
        Write the report to a file.
    to_html
        Return the report as an HTML string.
    """
    html = ''
    file = None

    def __init__(self, df, **kwargs):
        """Constructor see class documentation
        """
        sample = kwargs.get('sample', df.head())

        description_set = describe(df, **kwargs)

        self.html = to_html(sample,
                            description_set)

        self.description_set = description_set

    def get_description(self):
        """Return the description (a raw statistical summary) of the dataset.

        Returns
        -------
        dict
            Containing the following keys:
                * table: general statistics on the dataset
                * variables: summary statistics for each variable
                * freq: frequency table
        """
        return self.description_set

    def get_rejected_variables(self, threshold=0.9):
        """Return a list of variable names being rejected for high
        correlation with one of remaining variables.

        Parameters:
        ----------
        threshold : float
            Correlation value which is above the threshold are rejected
        
        Returns
        -------
        list
            The list of rejected variables or an empty list if the correlation has not been computed.
        """
        variable_profile = self.description_set['variables']
        result = []
        if hasattr(variable_profile, 'correlation'):
            result = variable_profile.index[variable_profile.correlation > threshold].tolist()
        return  result

    def to_file(self, outputfile=DEFAULT_OUTPUTFILE):
        """Write the report to a file.
        
        By default a name is generated.

        Parameters:
        ----------
        outputfile : str
            The name or the path of the file to generale including the extension (.html).
        """
    
        if outputfile != NO_OUTPUTFILE:
            if outputfile == DEFAULT_OUTPUTFILE:
                outputfile = 'profile_' + str(hash(self)) + ".html"
            # TODO: should be done in the template
            with codecs.open(outputfile, 'w+b', encoding='utf8') as self.file:
                self.file.write(templates.template('wrapper').render(content=self.html))

    def to_html(self):
        """Generate and return complete template as lengthy string
            for using with frameworks.
        
        Returns
        -------
        str
            The HTML output.
        """
        return templates.template('wrapper').render(content=self.html)

    def _repr_html_(self):
        """Used to output the HTML representation to a Jupyter notebook
        
        Returns
        -------
        str
            The HTML internal representation.
        """
        return self.html

    def __str__(self):
        """Overwrite of the str method.
        
        Returns
        -------
        str
            A string representation of the object.
        """
        return "Output written to file " + str(self.file.name)



# coding=UTF-8

import unittest

import datetime
import numpy as np
import pandas as pd
from pandas import Series
import six
import pandas_profiling
from pandas_profiling.describe import describe, describe_1d
from pandas_profiling.report import to_html
import tempfile
import shutil
import os
check_is_NaN = "pandas_profiling.check_is_NaN"


class DataFrameTest(unittest.TestCase):

    def setUp(self):
        self.data = {
            'id': [chr(97 + c) for c in range(1, 10)],
            'x': [50, 50, -10, 0, 0, 5, 15, -3, None],
            'y': [0.000001, 654.152, None, 15.984512, 3122, -3.1415926535, 111, 15.9, 13.5],
            'cat': ['a', 'long text value', u'Élysée', '', None, 'some <b> B.s </div> </div> HTML stuff', 'c', 'c', 'c'],
            's1': np.ones(9),
            's2': [u'some constant text $ % value {obj} ' for _ in range(1, 10)],
            'somedate': [datetime.date(2011, 7, 4), datetime.datetime(2022, 1, 1, 13, 57), datetime.datetime(1990, 12, 9), np.nan, datetime.datetime(1990, 12, 9), datetime.datetime(1950, 12, 9), datetime.datetime(1898, 1, 2), datetime.datetime(1950, 12, 9), datetime.datetime(1950, 12, 9)],
            'bool_tf': [True, True, False, True, False, True, True, False, True],
            'bool_tf_with_nan': [True, False, False, False, False, True, True, False, np.nan],
            'bool_01': [1, 1, 0, 1, 1, 0, 0, 0, 1],
            'bool_01_with_nan': [1, 0, 1, 0, 0, 1, 1, 0, np.nan],
            'list': [[1, 2], [1, 2], [1, 2], [1, 2], [1, 2], [1, 2], [1, 2], [1, 2], [1, 2]],
            'mixed': [1, 2, "a", 4, 5, 6, 7, 8, 9],
            'dict': [{'a':'a'}, {'b': 'b'}, {'c': 'c'}, {'d': 'd'}, {'e': 'e'}, {'f': 'f'}, {'g': 'g'}, {'h': 'h'}, {'i': 'i'}],
            'tuple': [(1,2), (3, 4), (5, 6), (7, 8), (9, 10), (11, 12), (13, 14), (15, 16), (17, 18)]
        }
        self.df = pd.DataFrame(self.data)
        self.df['somedate'] = pd.to_datetime(self.df['somedate'])

        self.results = describe(self.df)
        self.test_dir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.test_dir)

    def test_describe_df(self):

        expected_results = {}
        
        expected_results['id'] = {
            '25%': check_is_NaN, '5%': check_is_NaN, '50%': check_is_NaN, '75%': check_is_NaN, '95%': check_is_NaN, 'count': 9, 'n_infinite': 0, 'p_infinite': 0,
            'cv': check_is_NaN, 'distinct_count': 9, 'freq': check_is_NaN, 'histogram': check_is_NaN, 'iqr': check_is_NaN,
            'is_unique': True, 'kurtosis': check_is_NaN, 'mad': check_is_NaN, 'max': check_is_NaN, 'mean': check_is_NaN,
            'min': check_is_NaN, 'mini_histogram': check_is_NaN, 'n_missing': 0, 'p_missing': 0.0,
            'p_unique': 1.0, 'p_zeros': check_is_NaN, 'range': check_is_NaN, 'skewness': check_is_NaN,
            'std': check_is_NaN, 'sum': check_is_NaN, 'top': check_is_NaN, 'type': 'UNIQUE', 'variance': check_is_NaN
        }

        expected_results['x'] = {
            '25%': -0.75, '5%': -7.5499999999999989, '50%': 2.5, '75%': 23.75, '95%': 50.0,
            'count': 8, 'n_infinite': 0, 'p_infinite': 0, 'cv': 1.771071190261633, 'distinct_count': 7, 'freq': check_is_NaN, 'iqr': 24.5,
            'is_unique': False, 'kurtosis': -0.50292858929003803, 'mad': 18.71875, 'max': 50.0,
            'mean': 13.375, 'min': -10.0, 'mode': 0.0, 'n_missing': 1,
            'p_missing': 0.11111111111111116, 'p_unique': 7/9, 'p_zeros': 0.2222222222222222,
            'range': 60.0, 'skewness': 1.0851622393567653, 'std': 23.688077169749342, 'sum': 107.0,
            'top': check_is_NaN, 'type': 'NUM', 'variance': 561.125
        }

        expected_results['y'] = {
            '25%': 10.125000249999999, '5%': -2.0420348747749997, '50%': 15.942256,
            '75%': 246.78800000000001, '95%': 2258.2531999999987, 'count': 8, 'n_infinite': 0, 'p_infinite': 0,
            'cv': 2.2112992878833846, 'distinct_count': 9, 'freq': check_is_NaN,
            'iqr': 236.66299975000001, 'is_unique': True, 'kurtosis': 6.974137018717359,
            'mad': 698.45081747834365, 'max': 3122.0, 'mean': 491.17436504331249,
            'min': -3.1415926535000001, 'mode': 9.9999999999999995e-07, 'n_missing': 1,
            'p_missing': 0.11111111111111116, 'p_unique': 1, 'p_zeros': 0.0,
            'range': 3125.1415926535001, 'skewness': 2.6156591135729266, 'std': 1086.1335236468506,
            'sum': 3929.3949203464999, 'top': check_is_NaN, 'type': 'NUM',
            'variance': 1179686.0311895239
        }

        expected_results['cat'] = {
            '25%': check_is_NaN, '5%': check_is_NaN, '50%': check_is_NaN, '75%': check_is_NaN, '95%': check_is_NaN, 'count': 8, 'n_infinite': 0, 'p_infinite': 0,
            'cv': check_is_NaN, 'distinct_count': 7, 'freq': 3, 'histogram': check_is_NaN, 'iqr': check_is_NaN,
            'is_unique': False, 'kurtosis': check_is_NaN, 'mad': check_is_NaN, 'max': check_is_NaN, 'mean': check_is_NaN,
            'min': check_is_NaN, 'mini_histogram': check_is_NaN, 'mode': 'c',
            'n_missing': 1, 'p_missing': 0.11111111111111116, 'p_unique': 7/9,
            'p_zeros': check_is_NaN, 'range': check_is_NaN, 'skewness': check_is_NaN, 'std': check_is_NaN, 'sum': check_is_NaN,
            'top': 'c', 'type': 'CAT', 'variance': check_is_NaN
        }

        expected_results['s1'] = {
            '25%': check_is_NaN, '5%': check_is_NaN, '50%': check_is_NaN, '75%': check_is_NaN, '95%': check_is_NaN, 'count': 9, 'n_infinite': 0, 'p_infinite': 0,
            'cv': check_is_NaN, 'distinct_count': 1, 'freq': check_is_NaN, 'histogram': check_is_NaN, 'iqr': check_is_NaN,
            'is_unique': False, 'kurtosis': check_is_NaN, 'mad': check_is_NaN, 'max': check_is_NaN, 'mean': check_is_NaN,
            'min': check_is_NaN, 'mini_histogram': check_is_NaN, 'mode': 1.0,
            'n_missing': 0, 'p_missing': 0.0, 'p_unique': 0.1111111111111111, 'p_zeros': check_is_NaN,
            'range': check_is_NaN, 'skewness': check_is_NaN, 'std': check_is_NaN, 'sum': check_is_NaN, 'top': check_is_NaN,
            'type': 'CONST', 'variance': check_is_NaN
        }

        expected_results['s2'] = {
            '25%': check_is_NaN, '5%': check_is_NaN, '50%': check_is_NaN, '75%': check_is_NaN, '95%': check_is_NaN, 'count': 9, 'n_infinite': 0, 'p_infinite': 0,
            'cv': check_is_NaN, 'distinct_count': 1, 'freq': check_is_NaN, 'histogram': check_is_NaN, 'iqr': check_is_NaN,
            'is_unique': False, 'kurtosis': check_is_NaN, 'mad': check_is_NaN, 'max': check_is_NaN, 'mean': check_is_NaN,
            'min': check_is_NaN, 'mini_histogram': check_is_NaN,
            'mode': u'some constant text $ % value {obj} ', 'n_missing': 0, 'p_missing': 0.0,
            'p_unique': 0.1111111111111111, 'p_zeros': check_is_NaN, 'range': check_is_NaN,
            'skewness': check_is_NaN, 'std': check_is_NaN, 'sum': check_is_NaN, 'top': check_is_NaN, 'type': 'CONST',
            'variance': check_is_NaN
        }

        expected_results['somedate'] = {
            '25%': check_is_NaN, '5%': check_is_NaN, '50%': check_is_NaN, '75%': check_is_NaN, '95%': check_is_NaN,
            'count': 8, 'n_infinite': 0, 'p_infinite': 0, 'cv': check_is_NaN, 'distinct_count': 6, 'freq': check_is_NaN,
            'iqr': check_is_NaN, 'is_unique': False, 'kurtosis': check_is_NaN,
            'mad': check_is_NaN, 'max': datetime.datetime(2022, 1, 1, 13, 57), 'mean': check_is_NaN,
            'min': datetime.datetime(1898, 1, 2),
            'mode': datetime.datetime(1950, 12, 9),
            'n_missing': 1, 'p_missing': 0.11111111111111116, 'p_unique': 6/9,
            'p_zeros': check_is_NaN, 'range': datetime.timedelta(45289, hours=13, minutes=57),
            'skewness': check_is_NaN, 'std': check_is_NaN, 'sum': check_is_NaN, 'top': check_is_NaN, 'type': 'DATE',
        }

        expected_results['bool_tf'] = {
            '25%': check_is_NaN, '5%': check_is_NaN, '50%': check_is_NaN, '75%': check_is_NaN,
            '95%': check_is_NaN, 'count': 9, 'n_infinite': 0, 'p_infinite': 0,
            'cv': check_is_NaN, 'distinct_count': 2, 'freq': 6, 'histogram': check_is_NaN,
            'iqr': check_is_NaN,
            'is_unique': False, 'kurtosis': check_is_NaN, 'mad': check_is_NaN,
            'max': check_is_NaN, 'mean': 2 / 3,
            'min': check_is_NaN, 'mini_histogram': check_is_NaN, 'mode': True,
            'n_missing': 0, 'p_missing': 0, 'p_unique': 2 / 9,
            'p_zeros': check_is_NaN, 'range': check_is_NaN, 'skewness': check_is_NaN,
            'std': check_is_NaN, 'sum': check_is_NaN,
            'top': True, 'type': 'BOOL', 'variance': check_is_NaN
        }

        expected_results['bool_tf_with_nan'] = {
            '25%': check_is_NaN, '5%': check_is_NaN, '50%': check_is_NaN,
            '75%': check_is_NaN,
            '95%': check_is_NaN, 'count': 8, 'n_infinite': 0, 'p_infinite': 0,
            'cv': check_is_NaN, 'distinct_count': 3, 'freq': 5, 'histogram': check_is_NaN,
            'iqr': check_is_NaN,
            'is_unique': False, 'kurtosis': check_is_NaN, 'mad': check_is_NaN,
            'max': check_is_NaN, 'mean': check_is_NaN,
            'min': check_is_NaN, 'mini_histogram': check_is_NaN, 'mode': False,
            'n_missing': 1, 'p_missing': 0.11111111111111116, 'p_unique': 3 / 9,
            'p_zeros': check_is_NaN, 'range': check_is_NaN, 'skewness': check_is_NaN,
            'std': check_is_NaN, 'sum': check_is_NaN,
            'top': False, 'type': 'CAT', 'variance': check_is_NaN
        }

        expected_results['bool_01'] = {
            '25%': check_is_NaN, '5%': check_is_NaN, '50%': check_is_NaN, '75%': check_is_NaN,
            '95%': check_is_NaN, 'count': 9, 'n_infinite': 0, 'p_infinite': 0,
            'cv': check_is_NaN, 'distinct_count': 2, 'freq': 5, 'histogram': check_is_NaN,
            'iqr': check_is_NaN,
            'is_unique': False, 'kurtosis': check_is_NaN, 'mad': check_is_NaN,
            'max': check_is_NaN, 'mean': 5 / 9,
            'min': check_is_NaN, 'mini_histogram': check_is_NaN, 'mode': True,
            'n_missing': 0, 'p_missing': 0, 'p_unique': 2 / 9,
            'p_zeros': check_is_NaN, 'range': check_is_NaN, 'skewness': check_is_NaN,
            'std': check_is_NaN, 'sum': check_is_NaN,
            'top': 1, 'type': 'BOOL', 'variance': check_is_NaN
        }

        expected_results['bool_01_with_nan'] = {
            '25%': 0.0, '5%': 0.0, '50%': 0.5,
            '75%': 1, '95%': 1, 'count': 8, 'n_infinite': 0, 'p_infinite': 0,
            'cv': 1.0690449676496976, 'distinct_count': 3, 'freq': check_is_NaN,
            'iqr': 1, 'is_unique': False, 'kurtosis': -2.8000000000000003,
            'mad': 0.5, 'max': 1, 'mean': 0.5,
            'min': 0, 'mode': 0.0, 'n_missing': 1,
            'p_missing': 0.11111111111111116, 'p_unique': 3 / 9, 'p_zeros': 4 / 9,
            'range': 1, 'skewness': 0, 'std': 	0.5345224838248488,
            'sum': 4, 'top': check_is_NaN, 'type': 'NUM',
            'variance': 0.2857142857142857
        }

        expected_results['list'] = {
            'count': 9, 'n_infinite': 0, 'p_infinite': 0,
            'n_missing': 0, 'p_missing': 0,
            'type': 'UNSUPPORTED',
        }

        expected_results['mixed'] = {
            'count': 9, 'n_infinite': 0, 'p_infinite': 0,
            'n_missing': 0, 'p_missing': 0,
            'type': 'UNIQUE',
        }

        expected_results['dict'] = {
            'count': 9, 'n_infinite': 0, 'p_infinite': 0,
            'n_missing': 0, 'p_missing': 0,
            'type': 'UNSUPPORTED',
        }

        expected_results['tuple'] = {
            'count': 9, 'n_infinite': 0, 'p_infinite': 0,
            'n_missing': 0, 'p_missing': 0,
            'type': 'UNSUPPORTED',
        }

        self.assertTrue(set({'table', 'variables', 'freq', 'correlations'}).issubset(set(self.results.keys())))
        self.assertSetEqual(
            set(self.results['freq'].keys()), set(self.data.keys()))
        self.assertSetEqual(
            set(self.results['variables'].index), set(self.data.keys()))
        print((self.results['table'].items()))
        self.assertTrue(set({
                                'CAT': 2,
                                'CONST': 2,
                                'DATE': 1,
                                'NUM': 3,
                                'UNIQUE': 2,
                                'BOOL': 2,
                                'REJECTED': 2,
                                'RECODED': 0,
                                'CORR': 0,
                                'UNSUPPORTED': 3,
                                'n': 9,
                                'nvar': 15,
                                'n_duplicates': 0
                               }.items()).issubset(set(self.results['table'].items())))

        self.assertAlmostEqual(0.044444444444444446,
                               self.results['table']['total_missing'], 7)
        # Loop over variables
        for col in self.data.keys():
            for k, v in six.iteritems(expected_results[col]):
                if v == check_is_NaN:
                    self.assertTrue(np.isnan(self.results['variables'].loc[col][k]), msg="Value {} for key {} in column {} is not NaN".format(
                        self.results['variables'].loc[col][k], k, col))
                elif isinstance(v, float):
                    self.assertAlmostEqual(
                        v, self.results['variables'].loc[col][k], 7, msg="Value {} for key {} in column {} is not NaN".format(
                        self.results['variables'].loc[col][k], k, col))
                else:
                    self.assertEqual(v, self.results['variables'].loc[col][k], msg="Value {} for key {} in column {} is not NaN".format(
                        self.results['variables'].loc[col][k], k, col))

            if self.results['variables'].loc[col]['type'] in ['NUM', 'DATE']:
                self.assertLess(200, len(self.results['variables'].loc[col]["histogram"]),
                                "Histogram missing for column %s " % col)
                self.assertLess(200, len(self.results['variables'].loc[col]["mini_histogram"]),
                                "Mini-histogram missing for column %s " % col)

    def test_html_report(self):
        html = to_html(self.df.head(), self.results)
        self.assertLess(1000, len(html))

    def test_bins(self):
        self.results = describe(self.df, bins=100)
        self.test_describe_df()

    def test_export_to_file(self):

        p = pandas_profiling.ProfileReport(self.df)
        filename = os.path.join(self.test_dir, "profile_%s.html" % hash(self))
        p.to_file(outputfile=filename)

        self.assertLess(200, os.path.getsize(filename))


class CategoricalDataTest(unittest.TestCase):

    def test_recoding_reject(self):
        self.data = {
             'x': ['chien', 'chien', 'chien', 'chien', 'chat', 'chat', 'chameaux', 'chameaux'],
             'y': ['dog', 'dog', 'dog', 'dog', 'cat', 'cat', 'camel', 'camel'],
           }
        self.df = pd.DataFrame(self.data)
        self.results = describe(self.df)

        self.assertEqual(self.results['variables'].loc['x']['type'], 'RECODED')
        self.assertEqual(
            self.results['variables'].loc['x']['correlation_var'], 'y')

        expected_results = {'total_missing': 0.0, 'UNIQUE': 0, 'CONST': 0, 'nvar': 2, 'REJECTED': 1,
            'n': 8, 'RECODED': 1, 'CORR': 0, 'DATE': 0, 'NUM': 0, 'CAT': 1, 'n_duplicates': 5}
        for key in expected_results:
            self.assertEqual(self.results['table'][key], expected_results[key])

        # Rerun without checking for correlation
        self.results2 = describe(self.df, check_correlation=False)
        self.assertIsNone(
            self.results2['variables'].loc['x'].get('correlation_var'))
        self.assertEqual(self.results2['table']['REJECTED'], 0)


class Describe1dTest(unittest.TestCase):

    def test_unique(self):
        """Test the unique feature of 1D data"""
        # Unique values
        self._assert_unique(pd.Series([1, 2]), True, 1)
        # Unique values all nan
        self._assert_unique(pd.Series([np.nan]), True, 1)
        # Unique values including nan
        self._assert_unique(pd.Series([1, 2, np.nan]), True, 1)
        # Non unique values
        self._assert_unique(pd.Series([1, 2, 2]), False, 2/3)
        # Non unique nan
        self._assert_unique(pd.Series([1, np.nan, np.nan]), False, 2/3)
        # Non unique values including nan
        self._assert_unique(pd.Series([1, 2, 2, np.nan]), False, 3/4)
        # Non unique values including non unique nan
        self._assert_unique(pd.Series([1, 2, 2, np.nan, np.nan]), False, 3/5)

    def _assert_unique(self, data, is_unique, p_unique):
        desc_1d = describe_1d(data)
        self.assertEqual(desc_1d['is_unique'], is_unique)
        self.assertEqual(desc_1d['p_unique'], p_unique)

if __name__ == '__main__':
    unittest.main()



# coding=UTF-8
"""Formatters are utilities for formatting numbers by adding HTML code or CSS classes"""
import numpy as np

SKEWNESS_CUTOFF = 20
DEFAULT_FLOAT_FORMATTER = u'pandas_profiling.__default_float_formatter'


def gradient_format(value, limit1, limit2, c1, c2):
    def LerpColour(c1,c2,t):
        return (int(c1[0]+(c2[0]-c1[0])*t),int(c1[1]+(c2[1]-c1[1])*t),int(c1[2]+(c2[2]-c1[2])*t))
    c = LerpColour(c1, c2, (value-limit1)/(limit2-limit1))
    return fmt_color(value,"rgb{}".format(str(c)))


def fmt_color(text, color):
    return(u'<span style="color:{color}">{text}</span>'.format(color=color,text=str(text)))


def fmt_class(text, cls):
    return(u'<span class="{cls}">{text}</span>'.format(cls=cls,text=str(text)))


def fmt_bytesize(num, suffix='B'):
    for unit in ['','Ki','Mi','Gi','Ti','Pi','Ei','Zi']:
        if abs(num) < 1024.0:
            return "%3.1f %s%s" % (num, unit, suffix)
        num /= 1024.0
    return "%.1f %s%s" % (num, 'Yi', suffix)


def fmt_percent(v):
    return  "{:2.1f}%".format(v*100)

value_formatters={
    u'freq': (lambda v: gradient_format(v, 0, 62000, (30, 198, 244), (99, 200, 72))),
    u'p_missing': fmt_percent,
    u'p_infinite': fmt_percent,
    u'p_unique': fmt_percent,
    u'p_zeros': fmt_percent,
    u'memorysize': fmt_bytesize,
    u'total_missing': fmt_percent,
    DEFAULT_FLOAT_FORMATTER: lambda v: str(float('{:.5g}'.format(v))).rstrip('0').rstrip('.')
    }

def fmt_row_severity(v):
    if np.isnan(v) or v<= 0.01:
        return "ignore"
    else:
        return "alert"

def fmt_skewness(v):
    if not np.isnan(v) and (v<-SKEWNESS_CUTOFF or v> SKEWNESS_CUTOFF):
        return "alert"
    else:
        return ""

row_formatters={
    u'p_zeros': fmt_row_severity,
    u'p_missing': fmt_row_severity,
    u'p_infinite': fmt_row_severity,
    u'n_duplicates': fmt_row_severity,
    u'skewness': fmt_skewness,
}
